xquery version "3.1";

module namespace san="http://ahikar.sub.uni-goettingen.de/ns/annotations/save";

declare namespace http = "http://expath.org/ns/http-client";

import module namespace anno="http://ahikar.sub.uni-goettingen.de/ns/annotations" at "/db/apps/ahikar/modules/AnnotationAPI/annotations.xqm";
import module namespace commons="http://ahikar.sub.uni-goettingen.de/ns/commons" at "/db/apps/ahikar/modules/commons.xqm";
import module namespace requestr="http://exquery.org/ns/request";
import module namespace rest="http://exquery.org/ns/restxq";

declare namespace tei="http://www.tei-c.org/ns/1.0";

declare variable $san:server :=
    if(try {
        requestr:hostname() = "existdb"
    } catch * {
        true()
    })
        then $commons:expath-pkg/*/@name => replace("/$", "")
        else "http://localhost:8094/exist/restxq";

declare
    %rest:GET
    %rest:HEAD
    %rest:path("/annotations/save")
    %rest:query-param("token", "{$token}")
function san:main($token) {
    if( $token ne environment-variable("APP_DEPLOY_TOKEN" )) then
        error(QName("error://1", "deploy"), "Deploy token incorrect.")
    else
        let $sysout := util:log-system-out("Creating and storing the AnnotationCollections and AnnotationPages…")
        let $main-collection := map:keys($anno:uris)
        let $languages := map:get($anno:uris, $main-collection) => map:keys()
        for $lang in $languages return
            let $annotation-collection := local:save-annotation-collection($lang, (), ())
            let $manifests := map:get($anno:uris, $main-collection) => map:get($lang) => map:keys()
            for $manifest in $manifests return
                let $annotation-collection := local:save-annotation-collection($lang, $manifest, ())
                let $annotation-page := local:save-annotation-page($lang, $manifest, ())
                
                let $pages-in-manifest := 
                    for $tei in anno:find-in-map($anno:uris, $manifest) return
                        let $doc := commons:open-tei-xml($tei)
                        return
                            $doc//tei:pb[@facs]/@n/string()
                let $result :=
                    for $page in $pages-in-manifest return
                        (
                            local:save-annotation-collection($lang, $manifest, $page),
                            local:save-annotation-page($lang, $manifest, $page)
                        )
                return
                    util:log-system-out("Storing finished. For possible errors in this process see expath-repo.log.")
};

declare function local:save-annotation-collection($language as xs:string,
    $manifest as xs:string?,
    $page as xs:string?)
as xs:string {
    let $url := local:make-url($language, $manifest, $page, "annotationCollection")
    let $filename := local:make-filename($language, $manifest, $page, "annotationCollection")
    return
        local:make-and-store-request($url, $filename)
};

declare function local:make-url($language as xs:string,
    $manifest as xs:string?,
    $page as xs:string?,
    $type as xs:string)
as xs:string {
    if ($page) then
        "http://0.0.0.0:8080/exist/restxq/annotations/ahikar/" || $language || "/" || $manifest ||  "/" || $page || "/" || $type || ".json"
    else if ($manifest) then
        "http://0.0.0.0:8080/exist/restxq/annotations/ahikar/" || $language || "/" || $manifest || "/" || $type || ".json"
    else
        "http://0.0.0.0:8080/exist/restxq/annotations/ahikar/" || $language || "/" || $type || ".json"
};

declare function local:make-filename($language as xs:string,
    $manifest as xs:string?,
    $page as xs:string?,
    $type as xs:string)
as xs:string {
    if ($page) then
        $language || "-" || $manifest || "-" || $page || "-" || $type || ".json"
    else if ($manifest) then
        $language || "-" || $manifest || "-" || $type || ".json"
    else
        $language || "-" || $type || ".json"
};

declare function local:make-and-store-request($url as xs:string,
    $filename as xs:string)
as xs:string? {
    let $make-json-dir :=
        if (xmldb:collection-available($commons:json)) then
            ()
        else
            xmldb:create-collection("/db/data/textgrid", "json")
    return
        try {
            let $req := <http:request href="{$url}" method="get">
                    <http:header name="Connection" value="close"/>
               </http:request>
            let $json := 
                http:send-request($req)[2]
                => util:base64-decode()
            return
                xmldb:store-as-binary($commons:json, $filename, $json)
        } catch * {
            util:log("warn", "API request for " || $url || " not successful.")
        }
};

declare function local:save-annotation-page($language as xs:string,
    $manifest as xs:string,
    $page as xs:string?) {
    let $url := local:make-url ($language, $manifest, $page, "annotationPage")
    let $filename := local:make-filename ($language, $manifest, $page, "annotationPage")
    return
        local:make-and-store-request($url, $filename)
};
